@@ -24,7 +24,7 @@ use Try::Tiny;
 
 use Moo;
 use MooseX::CoverableModifiers;
-use Types::Standard qw/InstanceOf Str/;
+use Interchange6::Types -types;
 
 extends 'Interchange6::Cart';
 
@@ -115,6 +115,16 @@ sub _build_id {
     return shift->dbic_cart->id,
 }
 
+=head2 product_class
+
+Inherited. Change default to L<Dancer::Plugin::Interchange6::Cart::Product>.
+
+=cut
+
+has '+product_class' => (
+    default => 'Dancer::Plugin::Interchange6::Cart::Product',
+);
+
 =head2 sessions_id
 
 Extends inherited sessions_id attribute.
@@ -157,6 +167,7 @@ sub BUILD {
 
         push @products,
           {
+            dbic_product  => $record->product,
             id            => $record->cart_products_id,
             sku           => $record->sku,
             canonical_sku => $record->product->canonical_sku,
@@ -165,9 +176,6 @@ sub BUILD {
             price         => $record->product->price,
             uri           => $record->product->uri,
             weight        => $record->product->weight,
-            selling_price => $record->product->selling_price(
-                { quantity => $record->quantity, roles => $roles }
-            ),
           };
     }
 
@@ -225,15 +233,15 @@ around 'add' => sub {
           unless defined $result;
 
         my $product = {
+            dbic_product  => $result,
             name          => $result->name,
             price         => $result->price,
             sku           => $result->sku,
             canonical_sku => $result->canonical_sku,
             uri           => $result->uri,
             weight        => $result->weight,
+            quantity      => defined $arg->{quantity} ? $arg->{quantity} : 1,
         };
-        $product->{quantity} = $arg->{quantity}
-          if defined( $arg->{quantity} );
 
         push @products, $product;
     }
@@ -250,7 +258,7 @@ around 'add' => sub {
         # update or create in db
 
         my $cart_product =
-          $self->dbic_cart_products->search( { 'me.sku' => $product->{sku} },
+          $self->dbic_cart_products->search( { 'me.sku' => $ret->sku },
             { rows => 1 } )->single;
 
         if ( $cart_product ) {
@@ -266,14 +274,6 @@ around 'add' => sub {
             );
         }
 
-        # set selling_price
-
-        my $query = { quantity => $ret->quantity };
-        if ( logged_in_user ) {
-            $query->{roles} = [ user_roles ];
-        }
-        $ret->set_selling_price( $cart_product->product->selling_price($query) );
-
         push @ret, $ret;
     }
 
